import {
  IconButton,
  Paper,
  Table,
  TableBody,
  TableCell,
  TableContainer,
  TableRow,
  Tooltip,
  Typography,
} from "@mui/material";
import React from "react";
import DeleteIcon from "@mui/icons-material/Delete";
import { useParams } from "react-router-dom";
import { Device, DeviceApi } from "../api/DeviceApi";
import { AsyncWidget } from "../widgets/AsyncWidget";
import { SolarEnergyRouteContainer } from "../widgets/SolarEnergyRouteContainer";
import { useAlert } from "../hooks/context_providers/AlertDialogProvider";
import { useConfirm } from "../hooks/context_providers/ConfirmDialogProvider";
import { useLoadingMessage } from "../hooks/context_providers/LoadingMessageProvider";
import { useSnackbar } from "../hooks/context_providers/SnackbarProvider";

export function DeviceRoute(): React.ReactElement {
  const { id } = useParams();
  const [device, setDevice] = React.useState<Device | undefined>();

  const loadKey = React.useRef(1);

  const load = async () => {
    setDevice(await DeviceApi.GetSingle(id!));
  };

  const reload = () => {
    loadKey.current += 1;
    setDevice(undefined);
  };

  return (
    <AsyncWidget
      loadKey={loadKey.current}
      errMsg="Failed to load device information"
      load={load}
      ready={!!device}
      build={() => <DeviceRouteInner device={device!} onReload={reload} />}
    />
  );
}

function DeviceRouteInner(p: {
  device: Device;
  onReload: () => void;
}): React.ReactElement {
  const alert = useAlert();
  const confirm = useConfirm();
  const snackbar = useSnackbar();
  const loadingMessage = useLoadingMessage();

  const deleteDevice = async (d: Device) => {
    try {
      if (
        !(await confirm(
          `Do you really want to delete the device ${d.id}? The operation cannot be reverted!`
        ))
      )
        return;

      loadingMessage.show("Deleting device...");
      await DeviceApi.Delete(d);

      snackbar("The device has been successfully deleted!");
      p.onReload();
    } catch (e) {
      console.error(`Failed to delete device! ${e})`);
      alert("Failed to delete device!");
    } finally {
      loadingMessage.hide();
    }
  };
  return (
    <SolarEnergyRouteContainer
      label={`Device ${p.device.name}`}
      actions={
        <Tooltip title="Delete device">
          <IconButton onClick={() => deleteDevice(p.device)}>
            <DeleteIcon />
          </IconButton>
        </Tooltip>
      }
    >
      <GeneralDeviceInfo {...p} />
    </SolarEnergyRouteContainer>
  );
}

function GeneralDeviceInfo(p: { device: Device }): React.ReactElement {
  return (
    <TableContainer component={Paper}>
      <Typography variant="h6" style={{ padding: "6px" }}>
        General device information
      </Typography>
      <Table size="small">
        <TableBody>
          <DeviceInfoProperty label="ID" value={p.device.id} />
          <DeviceInfoProperty
            label="Reference"
            value={p.device.info.reference}
          />
          <DeviceInfoProperty label="Version" value={p.device.info.version} />
          <DeviceInfoProperty label="Name" value={p.device.name} />
          <DeviceInfoProperty
            label="Description"
            value={p.device.description}
          />
          <DeviceInfoProperty
            label="Enabled"
            value={p.device.enabled ? "YES" : "NO"}
          />
          <DeviceInfoProperty
            label="Maximum number of relays"
            value={p.device.info.max_relays.toString()}
          />
          <DeviceInfoProperty
            label="Number of configured relays"
            value={p.device.relays.length.toString()}
          />
        </TableBody>
      </Table>
    </TableContainer>
  );
}

function DeviceInfoProperty(p: {
  icon?: React.ReactElement;
  label: string;
  value: string;
}): React.ReactElement {
  return (
    <TableRow hover sx={{ "&:last-child td, &:last-child th": { border: 0 } }}>
      <TableCell>{p.label}</TableCell>
      <TableCell>{p.value}</TableCell>
    </TableRow>
  );
}
